(*------------------------------------------------------------------------
  TZipAdapter class.

  HISTORY

    Feb-19-2002 Initial version.

  Written by Huanlin Tsai.
------------------------------------------------------------------------*)
unit ZipAdapter;

interface

uses
  Windows, Messages, SysUtils, Classes, Forms, ZipMstr;

type
  TCompressLevel = (clStore, clFastest, clFast, clNormal, clGood, clBest);

  TAbstractArchiver = class(TObject)
  protected
    function GetArchiveName: string; virtual; abstract;
    procedure SetArchiveName(const Value: string); virtual; abstract;
    function GetBaseDir: string; virtual; abstract;
    procedure SetBaseDir(const Value: string); virtual; abstract;
    function GetCompressLevel: TCompressLevel; virtual; abstract;
    procedure SetCompressLevel(const Value: TCompressLevel); virtual; abstract;
    function GetSpecArgs: TStrings; virtual; abstract;
    function GetSubFolders: Boolean; virtual; abstract;
    procedure SetSubFolders(const Value: Boolean); virtual; abstract;
  public
    constructor Create; virtual; 

    procedure Add; virtual; abstract;     // Add file(s) to archive
    procedure Extract; virtual; abstract; // Extract file(s) from archive
    procedure ExtractTo(const DirName: string); virtual; abstract;

    property ArchiveName: string read GetArchiveName write SetArchiveName;
    property BaseDir: string read GetBaseDir write SetBaseDir;
    property SpecArgs: TStrings read GetSpecArgs;
    property SubFolders: Boolean read GetSubFolders write SetSubFolders;
    property CompressLevel: TCompressLevel read GetCompressLevel write SetCompressLevel;
  end;

  // Adaptor for TZipMaster (Delphi Zip)
  TZipAdapter = class(TAbstractArchiver)
  private
    FZip: TZipMaster;
    FBaseDir: string;
    FSubFolders: Boolean;
  protected
    function GetArchiveName: string; override;
    procedure SetArchiveName(const Value: string); override;
    function GetBaseDir: string; override;
    procedure SetBaseDir(const Value: string); override;
    function GetCompressLevel: TCompressLevel; override;
    procedure SetCompressLevel(const Value: TCompressLevel); override;
    function GetSpecArgs: TStrings; override;
    function GetSubFolders: Boolean; override;
    procedure SetSubFolders(const Value: Boolean); override;
  public
    constructor Create; override;
    destructor Destroy; override;

    procedure Add; override;
    procedure Extract; override;
    procedure ExtractTo(const DirName: string); override;
  end;

implementation

{$R ZipMsgTW.res}

{ TAbstractArchiver }

constructor TAbstractArchiver.Create;
begin
  inherited;
end;

{ TZipAdapter }

constructor TZipAdapter.Create;
begin
  inherited;
  FZip := TZipMaster.Create(nil);

  // set default properties
  SetCompressLevel(clBest);
  SetSubFolders(True);
end;

procedure TZipAdapter.Add;
begin
  if BaseDir = '' then
    BaseDir := '.';   // avoid bug of Delphi Zip v1.60p
  FZip.Add;
end;

procedure TZipAdapter.Extract;
begin
  FZip.Extract;
end;

procedure TZipAdapter.ExtractTo(const DirName: string);
begin
  BaseDir := DirName;
  Extract;
end;

destructor TZipAdapter.Destroy;
begin
  FreeAndNil(FZip);
  inherited;
end;

function TZipAdapter.GetBaseDir: string;
begin
  Result := FBaseDir;
end;

function TZipAdapter.GetArchiveName: string;
begin
  Result := FZip.ZipFileName;
end;

function TZipAdapter.GetSpecArgs: TStrings;
begin
  Result := FZip.FSpecArgs;
end;

function TZipAdapter.GetSubFolders: Boolean;
begin
  Result := FSubFolders;
end;

procedure TZipAdapter.SetBaseDir(const Value: string);
begin
  FBaseDir := Value;
  FZip.RootDir := Value;
  FZip.ExtrBaseDir := Value;
end;

procedure TZipAdapter.SetArchiveName(const Value: string);
begin
  FZip.ZipFileName := Value;
end;

procedure TZipAdapter.SetSubFolders(const Value: Boolean);
begin
  FSubFolders := Value;
  if FSubFolders then
  begin
    FZip.AddOptions := FZip.AddOptions + [AddDirNames, AddRecurseDirs];
    FZip.ExtrOptions := FZip.ExtrOptions + [ExtrDirNames];
  end
  else begin
    FZip.AddOptions := FZip.AddOptions - [AddDirNames, AddRecurseDirs];
    FZip.ExtrOptions := FZip.ExtrOptions - [ExtrDirNames];
  end;
end;

function TZipAdapter.GetCompressLevel: TCompressLevel;
begin
  case FZip.AddCompLevel of
    0:    Result := clStore;
    1, 2: Result := clFastest;
    3, 4: Result := clFast;
    5, 6: Result := clNormal;
    7, 8: Result := clGood;
    9:    Result := clBest;
  else
    raise Exception.Create('Invalid compression level');
  end;
end;

procedure TZipAdapter.SetCompressLevel(const Value: TCompressLevel);
begin
  case Value of
    clStore:    FZip.AddCompLevel := 0;
    clFastest:  FZip.AddCompLevel := 1;
    clFast:     FZip.AddCompLevel := 3;
    clNormal:   FZip.AddCompLevel := 5;
    clGood:     FZip.AddCompLevel := 7;
    clBest:     FZip.AddCompLevel := 9;
  end;
end;


end.
