(*----------------------------------------------------------------
  This unit contains definitions for an archiver interface and
  a factory class.

  Written by Huanlin Tsai.  Feb-19-2002.
----------------------------------------------------------------*)
unit AbsArchiver;

interface

uses
  Windows, SysUtils, Classes, Contnrs;

type
  TCompressLevel = (clStore, clFastest, clFast, clNormal, clGood, clBest);

  TAbstractArchiver = class(TObject)
  protected
    function GetArchiveName: string; virtual; abstract;
    procedure SetArchiveName(const Value: string); virtual; abstract;
    function GetBaseDir: string; virtual; abstract;
    procedure SetBaseDir(const Value: string); virtual; abstract;
    function GetCompressLevel: TCompressLevel; virtual; abstract;
    procedure SetCompressLevel(const Value: TCompressLevel); virtual; abstract;
    function GetSpecArgs: TStrings; virtual; abstract;
    function GetSubFolders: Boolean; virtual; abstract;
    procedure SetSubFolders(const Value: Boolean); virtual; abstract;
  public
    constructor Create; virtual; 

    procedure Add; virtual; abstract;     // Add file(s) to archive
    procedure Extract; virtual; abstract; // Extract file(s) from archive
    procedure ExtractTo(const DirName: string); virtual; abstract;

    property ArchiveName: string read GetArchiveName write SetArchiveName;
    property BaseDir: string read GetBaseDir write SetBaseDir;
    property SpecArgs: TStrings read GetSpecArgs;
    property SubFolders: Boolean read GetSubFolders write SetSubFolders;
    property CompressLevel: TCompressLevel read GetCompressLevel write SetCompressLevel;
  end;

  // Class reference type
  TArchiverClass = class of TAbstractArchiver;

  TArchiverClassMapping = class(TObject)
  private
    FMappingName: string;
    FArchiverClass: TArchiverClass;
  public
    constructor Create(const AMappingName: string; AClass: TArchiverClass);
    property MappingName: string read FMappingName;
    property ArchiverClass: TArchiverClass read FArchiverClass;
  end;

  TArchiverFactory = class(TObject)
  private
    FClasses: TObjectList;  // Stores class-mapping list
  protected
  public
    constructor Create;
    destructor Destroy; override;

    procedure RegisterClass(AClass: TArchiverClass);
    function CreateInstance(const AClassName: string): TAbstractArchiver; overload;
  end;

function ArchiverFactory: TArchiverFactory;

implementation

var
  g_ArchiverFactory: TArchiverFactory = nil;

function ArchiverFactory: TArchiverFactory;
begin
  if g_ArchiverFactory = nil then
    g_ArchiverFactory := TArchiverFactory.Create;
  Result := g_ArchiverFactory;
end;

{ TAbstractArchiver }

constructor TAbstractArchiver.Create;
begin
  inherited;
end;

{ TArchiverClassMapping }

constructor TArchiverClassMapping.Create(const AMappingName: string;
  AClass: TArchiverClass);
begin
  FMappingName := AMappingName;
  FArchiverClass := AClass;
end;

{ TArchiverFactory }

constructor TArchiverFactory.Create;
begin
  FClasses := TObjectList.Create;
end;

destructor TArchiverFactory.Destroy;
begin
  FClasses.Free;
  inherited;
end;

procedure TArchiverFactory.RegisterClass(AClass: TArchiverClass);
var
  i: integer;
  s1, s2: string;
begin
  s1 := AClass.ClassName;
  for i := 0 to FClasses.Count-1 do
  begin
    s2 := TArchiverClassMapping(FClasses.Items[i]).MappingName;
    if SameText(s1, s2) then  // pGOwgU
      Exit;                   //   N^
  end;
  FClasses.Add(TArchiverClassMapping.Create(s1, AClass));
end;

// Create an instance of TAbstractArchiver descendent
function TArchiverFactory.CreateInstance(
  const AClassName: string): TAbstractArchiver;
var
  i: integer;
  acm: TArchiverClassMapping;
begin
  for i := 0 to FClasses.Count - 1 do
  begin
    acm := TArchiverClassMapping(FClasses.Items[i]);
    if SameText(acm.MappingName, AClassName) then
    begin
      Result := acm.ArchiverClass.Create;
      Exit;
    end;
  end;

  raise Exception.CreateFmt('<%s> OU', [AClassName]) ;
end;

initialization

finalization
  if g_ArchiverFactory <> nil then
    FreeAndNil(g_ArchiverFactory);

end.
